////////////////////////////////////////////////////////////////////////////////
// Settings
////////////////////////////////////////////////////////////////////////////////

'use strict'

// Dependencies
const through = require('through2'),
      log = require('fancy-log'),
      chalk = require('chalk'),
      notification = require('gulp-notify');

module.exports.success = success;
module.exports.error = error;
module.exports.icon = icon;

////////////////////////////////////////////////////////////////////////////////
// Symbols File
////////////////////////////////////////////////////////////////////////////////


function icon() {
  if (this.cache) { return this.cache }
  try {
  	fs.accessSync(path.resolve(icon))
    return this.cache = icon;
  } catch(e){
    return this.cache = 'https://i.imgur.com/G6fTWAs.png';
  }
}

// This creates legibal messages depending on the environment settings.
function messages(message, extra) {

  let files = message;
  let action = versioning ? 'versionised' : 'compiled';
  var extra = extra && typeof extra !== 'undefined' ? extra + ' ' : '';

  switch(message) {
    case 'core':
      files = 'Core script';
      action = combineJS ? 'precompiled' : action;
    break;
    case 'js':
      files = combineJS ? 'Javascript and Core' : 'Javascript';
      action = combineJS ? (versioning ? 'combined and versionised' : 'combined') : action;
    break;
    case 'symbols':
      files = 'Symbol';
      action = 'compiled';
    break;
    case 'sass':
      files = 'SASS';
      action = combineCSS ? (versioning ? 'combined and versionised' : 'combined') : action;
    break;

  }

  return `${files} files ${action} ${extra}successfully`;

}

function error(error) {

  const line = error.loc ? error.loc.line : (error.line ? error.line : 'Unknown');
  const file = (error.fileName ? error.fileName : (error.file ? error.file : (error.relativePath ? error.relativePath : ''))).replace(process.cwd(), '');
  const path = error.relativePath || 'path';
  const task = error.plugin || 'Task unknown';
  const name = error.name || 'Name unknown';

  // Easy error reporting
  const warning = chalk.white.bgRed;
  const keyword = chalk.red.bgWhite;

  notification({
    icon: "https://i.imgur.com/VsfiLjV.png",
    title: `${project} : ${name}`,
    message: `Line ${line} in ${file}`,
  }).write(error);

  log(`${warning(" " +  task + " Error: ")} ${name} in ${file}: ${keyword(" line " + line + " ")} \n ${error.message ? error.message.replace(process.cwd(), '') : error.toString()}`);

  // Prevents any watchers from stopping
  this.emit('end');

}

function success(message, files, extra) {

  console.log('TRIGGER1', files);

  if (typeof files !== 'undefined') {
    files = typeof files == 'object' ? files : [files];
    files.forEach(file => {
      log(`${chalk.cyan("Created:")} ${chalk.green(file)}`);
    })
  }

  return notification({
    'icon': this.icon(),
    'title': project,
    'message': this.messages(message, extra),
    'onLast': true
  });

}
